@@ -3,6 +3,8 @@
 check_inclusion_prob <- function(x, n, s) {
   # sampling::inclusionprobabilities() gives a warning with 0s; I think
   # an error makes more sense
+  # e.g., x = c(0, 0, 1, 1) and n = 3 => units 1 and 2 have inclusion probs
+  # of 0, but at least one must be included in the sample
   if (not_strict_positive_vector(x)) {
     stop(
       gettext("'x' must be a strictly positive and finite numeric vector")
@@ -13,26 +15,28 @@ check_inclusion_prob <- function(x, n, s) {
       gettext("'n' must be a positive and finite numeric vector")
     )
   }
+  # needs to be the same length for tabulate()
   if (length(x) != length(s)) {
     stop(
-      gettext("'x' and 's' must be the same length")
+      gettext("'x' and 'strata' must be the same length")
     )
   }
   if (length(n) != nlevels(s)) {
     stop(
-      gettext("'n' must have a single sample size for each level in 's'")
+      gettext("'n' must have a single sample size for each level in 'strata'")
     )
   }
+  # missing strata means inclusion probs are all missing
   if (anyNA(s)) {
     stop(
-      gettext("'s' cannot contain NAs")
+      gettext("'strata' cannot contain NAs")
     )
   }
   # bins isn't wide enough by default to catch factors with unused levels
   # at the end
   if (any(tabulate(s, nbins = nlevels(s)) < n)) {
     stop(
-      gettext("sample size 'n' is greater than population size")
+      gettext("sample size 'n' is greater than population size for some strata")
     )
   }
 }
@@ -43,8 +47,10 @@ pi <- function(x, n) {
 
 .inclusion_prob <- function(x, n) {
   res <- pi(x, n)
-  if (length(res) == 0L || max(res) <= 1) return(res)
+  if (length(res) == 0L || max(res) <= 1) return(as.numeric(res))
   repeat {
+    # inclusion probs increase with each loop, so only need to recalculate
+    # those strictly less than 1
     keep_ts <- which(res < 1)
     n_ts <- n - length(x) + length(keep_ts)
     res[keep_ts] <- ts <- pi(x[keep_ts], n_ts)
@@ -53,25 +59,15 @@ pi <- function(x, n) {
   pmin.int(res, 1)
 }
 
-.inclusion_prob_list <- function(x, n, s) {
-  # the single stratum case is common enough to warrant the optimization
-  if (nlevels(s) == 1L) {
-    list(.inclusion_prob(x, n))
-  } else {
-    .mapply(.inclusion_prob, list(split(x, s), n), list())
-  }
-}
-
 #---- Inclusion probability ----
-inclusion_prob <- function(x, n, s = gl(1, length(x))) {
-  s <- as.factor(s)
+inclusion_prob <- function(x, n, strata = gl(1, length(x))) {
+  strata <- as.factor(strata)
   n <- trunc(n)
-  check_inclusion_prob(x, n, s)
-  res <- if (nlevels(s) == 1L) {
+  check_inclusion_prob(x, n, strata)
+  # the single stratum case is common enough to warrant the optimization
+  if (nlevels(strata) == 1L) {
     .inclusion_prob(x, n)
   } else {
-    unsplit(.inclusion_prob_list(x, n, s), s)
+    unsplit(Map(.inclusion_prob, split(x, strata), n), strata)
   }
-  attributes(res) <- NULL # unsplit() mangles attributes
-  res
 }
